@@ -30,10 +30,12 @@
 #include "ephy-embed-shell.h"
 #include "ephy-lib-type-builtins.h"
 #include "ephy-settings.h"
+#include "ephy-shell.h"
 #include "ephy-signal-accumulator.h"
 #include "ephy-suggestion.h"
 #include "ephy-title-widget.h"
 #include "ephy-uri-helpers.h"
+#include "ephy-web-view.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
@@ -114,13 +116,13 @@ static void ephy_location_entry_editable_init (GtkEditableInterface *iface);
 static void ephy_location_entry_accessible_init (GtkAccessibleInterface *iface);
 static void ephy_location_entry_title_widget_interface_init (EphyTitleWidgetInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_WIDGET,
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_EDITABLE,
-                                                ephy_location_entry_editable_init)
-                         G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE,
-                                                ephy_location_entry_accessible_init)
-                         G_IMPLEMENT_INTERFACE (EPHY_TYPE_TITLE_WIDGET,
-                                                ephy_location_entry_title_widget_interface_init))
+G_DEFINE_FINAL_TYPE_WITH_CODE (EphyLocationEntry, ephy_location_entry, GTK_TYPE_WIDGET,
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_EDITABLE,
+                                                      ephy_location_entry_editable_init)
+                               G_IMPLEMENT_INTERFACE (GTK_TYPE_ACCESSIBLE,
+                                                      ephy_location_entry_accessible_init)
+                               G_IMPLEMENT_INTERFACE (EPHY_TYPE_TITLE_WIDGET,
+                                                      ephy_location_entry_title_widget_interface_init))
 
 typedef struct {
   GUri *uri;
@@ -287,10 +289,27 @@ update_suggestions_popover (EphyLocationEntry *entry)
 
   n_items = g_list_model_get_n_items (G_LIST_MODEL (entry->suggestions_model));
 
-  if (entry->show_suggestions && n_items > 0)
+  if (entry->show_suggestions && n_items > 0) {
+    if (entry->adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW) {
+      GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (entry));
+      double offset;
+
+      gtk_widget_translate_coordinates (GTK_WIDGET (entry),
+                                        GTK_WIDGET (root),
+                                        0, 0,
+                                        &offset, NULL);
+
+      gtk_widget_set_halign (entry->suggestions_popover, GTK_ALIGN_START);
+      gtk_popover_set_offset (GTK_POPOVER (entry->suggestions_popover), -offset, 0);
+    } else {
+      gtk_widget_set_halign (entry->suggestions_popover, GTK_ALIGN_FILL);
+      gtk_popover_set_offset (GTK_POPOVER (entry->suggestions_popover), 0, 0);
+    }
+
     gtk_popover_popup (GTK_POPOVER (entry->suggestions_popover));
-  else
+  } else {
     gtk_popover_popdown (GTK_POPOVER (entry->suggestions_popover));
+  }
 }
 
 static void
@@ -1044,6 +1063,7 @@ ephy_location_entry_size_allocate (GtkWidget *widget,
   int icon_right_pos = width;
   GskTransform *transform;
   GList *l;
+  GtkWidget *root = GTK_WIDGET (gtk_widget_get_root (widget));
 
   for (l = entry->permission_buttons; l; l = l->next) {
     allocate_icon (widget, height, baseline, l->data,
@@ -1069,8 +1089,13 @@ ephy_location_entry_size_allocate (GtkWidget *widget,
 
   gtk_widget_allocate (entry->progress, width, height, baseline, NULL);
 
-  gtk_widget_set_size_request (entry->suggestions_popover,
-                               gtk_widget_get_allocated_width (widget), -1);
+  if (entry->adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW)
+    gtk_widget_set_size_request (entry->suggestions_popover,
+                                 gtk_widget_get_width (root), -1);
+  else
+    gtk_widget_set_size_request (entry->suggestions_popover,
+                                 gtk_widget_get_allocated_width (widget), -1);
+
   gtk_widget_queue_resize (entry->suggestions_popover);
 
   gtk_popover_present (GTK_POPOVER (entry->suggestions_popover));
@@ -1444,6 +1469,7 @@ ephy_location_entry_init (EphyLocationEntry *entry)
   entry->block_update = FALSE;
   entry->saved_text = NULL;
   entry->page_actions = NULL;
+  entry->adaptive_mode = EPHY_ADAPTIVE_MODE_NORMAL;
 
   gtk_widget_init_template (GTK_WIDGET (entry));
 
@@ -1898,7 +1924,7 @@ ephy_location_entry_get_reader_mode_state (EphyLocationEntry *entry)
   return entry->reader_mode_active;
 }
 
-static gboolean
+static void
 progress_hide (gpointer user_data)
 {
   EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
@@ -1907,11 +1933,9 @@ progress_hide (gpointer user_data)
   gtk_widget_set_visible (entry->progress, FALSE);
 
   g_clear_handle_id (&entry->progress_timeout, g_source_remove);
-
-  return G_SOURCE_REMOVE;
 }
 
-static gboolean
+static void
 ephy_location_entry_set_fraction_internal (gpointer user_data)
 {
   EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
@@ -1930,16 +1954,14 @@ ephy_location_entry_set_fraction_internal (gpointer user_data)
   progress = current + 0.025;
   if (progress < entry->progress_fraction) {
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (entry->progress), progress);
-    entry->progress_timeout = g_timeout_add (ms, ephy_location_entry_set_fraction_internal, entry);
+    entry->progress_timeout = g_timeout_add_once (ms, ephy_location_entry_set_fraction_internal, entry);
   } else {
     gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (entry->progress), entry->progress_fraction);
     if (entry->progress_fraction == 1.0)
-      entry->progress_timeout = g_timeout_add (500, progress_hide, entry);
+      entry->progress_timeout = g_timeout_add_once (500, progress_hide, entry);
   }
 
   gtk_widget_set_visible (entry->progress, TRUE);
-
-  return G_SOURCE_REMOVE;
 }
 
 void
@@ -1973,11 +1995,6 @@ ephy_location_entry_set_adaptive_mode (EphyLocationEntry *entry,
 {
   entry->adaptive_mode = adaptive_mode;
 
-  if (adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW)
-    gtk_widget_add_css_class (GTK_WIDGET (entry), "narrow");
-  else
-    gtk_widget_remove_css_class (GTK_WIDGET (entry), "narrow");
-
   ephy_location_entry_set_bookmark_icon_state (entry, entry->icon_state);
 }
 
