@@ -25,6 +25,7 @@
 #include "ephy-debug.h"
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
+#include "ephy-flatpak-utils.h"
 #include "ephy-profile-utils.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
@@ -36,13 +37,12 @@
 #include <glib/gi18n.h>
 #include <glib-unix.h>
 #include <gtk/gtk.h>
-#define HANDY_USE_UNSTABLE_API
-#include <handy.h>
 #include <libxml/xmlreader.h>
 #include <libxml/xmlversion.h>
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
+#include <sys/resource.h>
 
 static gboolean open_in_new_window = FALSE;
 
@@ -164,6 +164,28 @@ get_startup_id (void)
   return retval;
 }
 
+static void
+maximize_fd_limit (void)
+{
+  /* We need a relatively high number of file descriptors per web process. The
+   * default limit of 1024 will be exceeded with relatively small numbers of
+   * open tabs. Let's increase as far as we can. Note this code cannot safely
+   * go into WebKit because it will break applications that use select().
+   *
+   * https://gitlab.gnome.org/GNOME/epiphany/-/issues/2010
+   */
+  struct rlimit rlim;
+
+  if (getrlimit (RLIMIT_NOFILE, &rlim) == -1) {
+    g_warning ("Failed to read file descriptor limit: %s", g_strerror (errno));
+    return;
+  }
+
+  rlim.rlim_cur = rlim.rlim_max;
+  if (setrlimit (RLIMIT_NOFILE, &rlim) == -1)
+    g_warning ("Failed to set file descriptor limit: %s", g_strerror (errno));
+}
+
 int
 main (int   argc,
       char *argv[])
@@ -183,6 +205,8 @@ main (int   argc,
   g_setenv ("GSETTINGS_SCHEMA_DIR", BUILD_ROOT "/data", FALSE);
 #endif
 
+  g_setenv ("PULSE_PROP_application.icon_name", APPLICATION_ID, TRUE);
+
   /* Initialize the i18n stuff */
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -262,8 +286,6 @@ main (int   argc,
 
   g_option_context_set_main_group (option_context, option_group);
 
-  g_option_context_add_group (option_context, gtk_get_option_group (TRUE));
-
   if (!g_option_context_parse (option_context, &argc, &argv, &error)) {
     g_print ("Failed to parse arguments: %s\n", error->message);
     g_error_free (error);
@@ -296,14 +318,19 @@ main (int   argc,
 
   if (application_mode && !profile_directory) {
     if (desktop_file_basename) {
-      desktop_info = g_desktop_app_info_new (desktop_file_basename);
-
-      if (desktop_info)
-        profile_directory = ephy_web_application_ensure_for_app_info (G_APP_INFO (desktop_info));
-
-      if (!profile_directory) {
-        g_print ("Invalid desktop file passed to --application-mode\n");
+      if (ephy_is_running_inside_sandbox ()) {
+        g_print ("In sandbox, no desktop file can be passed to --application-mode\n");
         exit (1);
+      } else {
+        desktop_info = g_desktop_app_info_new (desktop_file_basename);
+
+        if (desktop_info)
+          profile_directory = ephy_web_application_ensure_for_app_info (G_APP_INFO (desktop_info));
+
+        if (!profile_directory) {
+          g_print ("Invalid desktop file passed to --application-mode\n");
+          exit (1);
+        }
       }
     }
   }
@@ -323,8 +350,6 @@ main (int   argc,
     flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
   if (incognito_mode)
     flags |= EPHY_FILE_HELPERS_STEAL_DATA;
-  if (profile_directory && !incognito_mode)
-    flags |= EPHY_FILE_HELPERS_KEEP_DIR;
 
   if (!ephy_file_helpers_init (profile_directory, flags, &error)) {
     g_error ("Fatal initialization error: %s", error->message);
@@ -374,7 +399,7 @@ main (int   argc,
   /* Delete the requested web application, if any. Must happen after
    * ephy_file_helpers_init (). */
   if (application_to_delete) {
-    ephy_web_application_delete (application_to_delete);
+    ephy_web_application_delete (application_to_delete, NULL);
     exit (0);
   }
 
@@ -409,6 +434,8 @@ main (int   argc,
     gtk_window_set_default_icon_name (APPLICATION_ID);
   }
 
+  maximize_fd_limit ();
+
   _ephy_shell_create_instance (mode);
 
   if (search_term) {
@@ -431,9 +458,12 @@ main (int   argc,
 
   status = g_application_run (G_APPLICATION (ephy_shell), argc, argv);
 
-  /* Shutdown */
-  /* FIXME: should free this stuff even when exiting early */
+  /* Ensure the EphyShell really gets destroyed. This should be the final reference. */
+  g_object_add_weak_pointer (G_OBJECT (ephy_shell), (gpointer *)&ephy_shell);
   g_object_unref (ephy_shell);
+  g_assert (!ephy_shell);
+
+  /* FIXME: should free this stuff even when exiting early */
   g_free (desktop_file_basename);
   g_free (profile_directory);
 
