@@ -34,10 +34,11 @@ struct _EphyTabView {
 
   AdwTabView *tab_view;
   AdwTabBar *tab_bar;
+  AdwTabOverview *tab_overview;
   AdwTabPage *current_page;
 };
 
-G_DEFINE_TYPE (EphyTabView, ephy_tab_view, ADW_TYPE_BIN)
+G_DEFINE_FINAL_TYPE (EphyTabView, ephy_tab_view, ADW_TYPE_BIN)
 
 enum {
   PROP_0,
@@ -397,6 +398,20 @@ update_icon_cb (AdwTabPage *page)
   adw_tab_page_set_icon (page, placeholder_icon);
 }
 
+static void
+update_uri_cb (AdwTabPage *page)
+{
+  EphyEmbed *embed = EPHY_EMBED (adw_tab_page_get_child (page));
+  EphyWebView *view = ephy_embed_get_web_view (embed);
+  const char *uri;
+
+  update_icon_cb (page);
+
+  uri = webkit_web_view_get_uri (WEBKIT_WEB_VIEW (view));
+
+  adw_tab_page_set_keyword (page, uri);
+}
+
 static void
 update_indicator_cb (AdwTabPage *page)
 {
@@ -414,6 +429,17 @@ update_indicator_cb (AdwTabPage *page)
   adw_tab_page_set_indicator_icon (page, icon);
 }
 
+gboolean
+is_loading_transform_cb (GBinding     *binding,
+                         const GValue *from_value,
+                         GValue       *to_value,
+                         gpointer      user_data)
+{
+  EphyEmbed *embed = user_data;
+  g_value_set_boolean (to_value, g_value_get_boolean (from_value) && !ephy_embed_has_load_pending (embed));
+  return TRUE;
+}
+
 int
 ephy_tab_view_add_tab (EphyTabView *self,
                        EphyEmbed   *embed,
@@ -442,7 +468,7 @@ ephy_tab_view_add_tab (EphyTabView *self,
 
   adw_tab_page_set_indicator_activatable (page, TRUE);
 
-  g_object_bind_property (view, "is-loading", page, "loading", G_BINDING_SYNC_CREATE);
+  g_object_bind_property_full (view, "is-loading", page, "loading", G_BINDING_SYNC_CREATE, is_loading_transform_cb, NULL, embed, NULL);
 
   g_signal_connect_object (embed, "notify::title",
                            G_CALLBACK (update_title_cb), page,
@@ -454,7 +480,7 @@ ephy_tab_view_add_tab (EphyTabView *self,
                            G_CALLBACK (update_icon_cb), page,
                            G_CONNECT_SWAPPED);
   g_signal_connect_object (view, "notify::uri",
-                           G_CALLBACK (update_icon_cb), page,
+                           G_CALLBACK (update_uri_cb), page,
                            G_CONNECT_SWAPPED);
   g_signal_connect_object (view, "notify::is-playing-audio",
                            G_CALLBACK (update_indicator_cb), page,
@@ -464,7 +490,7 @@ ephy_tab_view_add_tab (EphyTabView *self,
                            G_CONNECT_SWAPPED);
 
   update_title_cb (page);
-  update_icon_cb (page);
+  update_uri_cb (page);
   update_indicator_cb (page);
 
   return adw_tab_view_get_page_position (self->tab_view, page);
@@ -519,7 +545,7 @@ drag_drop_cb (EphyTabView  *self,
     g_auto (GStrv) split = g_strsplit (text, "\n", MAX_NUMBER_OF_URLS);
     int i;
 
-    for (i = 0; split[i] && *split[i] != '\0'; i++) {
+    for (i = 0; *split[i]; i++) {
       const char *uri = split[i];
       g_autofree char *normalized =
         ephy_embed_utils_normalize_or_autosearch_address (uri);
@@ -583,7 +609,13 @@ is_layout_reversed (void)
 static void
 notify_decoration_layout_cb (EphyTabView *self)
 {
-  adw_tab_bar_set_inverted (self->tab_bar, is_layout_reversed ());
+  gboolean inverted = is_layout_reversed ();
+
+  if (self->tab_bar)
+    adw_tab_bar_set_inverted (self->tab_bar, inverted);
+
+  if (self->tab_overview)
+    adw_tab_overview_set_inverted (self->tab_overview, inverted);
 }
 
 void
@@ -640,3 +672,22 @@ ephy_tab_view_set_tab_bar (EphyTabView *self,
   visibility_policy_changed_cb (self);
   notify_decoration_layout_cb (self);
 }
+
+void
+ephy_tab_view_set_tab_overview (EphyTabView    *self,
+                                AdwTabOverview *tab_overview)
+{
+  self->tab_overview = tab_overview;
+
+  adw_tab_overview_setup_extra_drop_target (tab_overview, GDK_ACTION_COPY, (GType[3]) {
+    G_TYPE_STRING,
+    G_TYPE_FILE,
+    GDK_TYPE_FILE_LIST,
+  }, 3);
+
+  g_signal_connect_object (tab_overview, "extra-drag-drop",
+                           G_CALLBACK (drag_drop_cb), self,
+                           G_CONNECT_SWAPPED);
+
+  notify_decoration_layout_cb (self);
+}
