@@ -30,6 +30,7 @@
 #include "ephy-langs.h"
 #include "ephy-settings.h"
 #include "ephy-search-engine-listbox.h"
+#include "ephy-shell.h"
 #include "ephy-web-app-utils.h"
 #include "webapp-additional-urls-dialog.h"
 
@@ -45,7 +46,6 @@ struct _PrefsGeneralPage {
   AdwPreferencesPage parent_instance;
 
   /* Web Application */
-  EphyWebApplication *webapp;
   guint webapp_save_id;
   GtkWidget *webapp_box;
   GtkWidget *webapp_icon;
@@ -54,8 +54,8 @@ struct _PrefsGeneralPage {
   GtkWidget *webapp_title_row;
 
   /* Web Content */
-  GtkWidget *adblock_allow_switch;
-  GtkWidget *popups_allow_switch;
+  GtkWidget *adblock_allow_row;
+  GtkWidget *popups_allow_row;
 
   /* Homepage */
   GtkWidget *homepage_box;
@@ -66,7 +66,7 @@ struct _PrefsGeneralPage {
 
   /* Downloads */
   GtkWidget *download_box;
-  GtkWidget *ask_on_download_switch;
+  GtkWidget *ask_on_download_row;
   GtkWidget *download_folder_row;
   GtkWidget *download_folder_label;
 
@@ -75,19 +75,18 @@ struct _PrefsGeneralPage {
 
   /* Session */
   GtkWidget *session_box;
-  GtkWidget *start_in_incognito_mode_switch;
+  GtkWidget *start_in_incognito_mode_row;
   GtkWidget *restore_session_row;
-  GtkWidget *restore_session_switch;
 
   /* Browsing */
   GtkWidget *browsing_box;
-  GtkWidget *enable_mouse_gesture_switch;
+  GtkWidget *enable_mouse_gesture_row;
   GtkWidget *enable_switch_to_new_tab;
 
   /* Languages */
   AdwPreferencesGroup *lang_group;
   GtkWidget *lang_listbox;
-  GtkWidget *enable_spell_checking_switch;
+  GtkWidget *enable_spell_checking_row;
 
   GtkWindow *add_lang_dialog;
   GtkTreeView *add_lang_treeview;
@@ -95,7 +94,7 @@ struct _PrefsGeneralPage {
   GCancellable *cancellable;
 };
 
-G_DEFINE_TYPE (PrefsGeneralPage, prefs_general_page, ADW_TYPE_PREFERENCES_PAGE)
+G_DEFINE_FINAL_TYPE (PrefsGeneralPage, prefs_general_page, ADW_TYPE_PREFERENCES_PAGE)
 
 static void
 prefs_general_page_dispose (GObject *object)
@@ -115,7 +114,6 @@ prefs_general_page_dispose (GObject *object)
     g_object_unref (general_page->add_lang_dialog);
   }
 
-  g_clear_pointer (&general_page->webapp, ephy_web_application_free);
   G_OBJECT_CLASS (prefs_general_page_parent_class)->dispose (object);
 }
 
@@ -609,35 +607,40 @@ save_web_application (PrefsGeneralPage *general_page)
 {
   gboolean changed = FALSE;
   const char *text;
+  EphyWebApplication *webapp = ephy_shell_get_webapp (ephy_shell_get_default ());
 
   general_page->webapp_save_id = 0;
 
-  if (!general_page->webapp)
+  if (!webapp)
     return G_SOURCE_REMOVE;
 
   text = gtk_editable_get_text (GTK_EDITABLE (general_page->webapp_url_row));
-  if (g_strcmp0 (general_page->webapp->url, text) != 0) {
-    g_free (general_page->webapp->url);
-    general_page->webapp->url = g_strdup (text);
+  if (g_strcmp0 (webapp->url, text) != 0) {
+    g_free (webapp->url);
+    webapp->url = g_strdup (text);
     changed = TRUE;
   }
 
   text = gtk_editable_get_text (GTK_EDITABLE (general_page->webapp_title_row));
-  if (g_strcmp0 (general_page->webapp->name, text) != 0) {
-    g_free (general_page->webapp->name);
-    general_page->webapp->name = g_strdup (text);
+  if (g_strcmp0 (webapp->name, text) != 0) {
+    g_free (webapp->name);
+    webapp->name = g_strdup (text);
     changed = TRUE;
   }
 
   text = (const char *)g_object_get_data (G_OBJECT (general_page->webapp_icon), "ephy-webapp-icon-path");
-  if (g_strcmp0 (general_page->webapp->icon_path, text) != 0) {
-    g_free (general_page->webapp->icon_path);
-    general_page->webapp->icon_path = g_strdup (text);
+  if (g_strcmp0 (webapp->icon_path, text) != 0) {
+    g_free (webapp->icon_path);
+    webapp->icon_path = g_strdup (text);
     changed = TRUE;
   }
 
-  if (changed)
-    ephy_web_application_save (general_page->webapp);
+  if (changed) {
+    ephy_web_application_save (webapp);
+    ephy_shell_resync_title_boxes (ephy_shell_get_default (),
+                                   webapp->name,
+                                   webapp->url);
+  }
 
   return G_SOURCE_REMOVE;
 }
@@ -645,7 +648,8 @@ save_web_application (PrefsGeneralPage *general_page)
 static void
 prefs_general_page_save_web_application (PrefsGeneralPage *general_page)
 {
-  if (!general_page->webapp)
+  EphyWebApplication *webapp = ephy_shell_get_webapp (ephy_shell_get_default ());
+  if (!webapp)
     return;
 
   g_clear_handle_id (&general_page->webapp_save_id, g_source_remove);
@@ -912,8 +916,8 @@ prefs_general_page_class_init (PrefsGeneralPageClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, webapp_title_row);
 
   /* Web Content */
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, adblock_allow_switch);
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, popups_allow_switch);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, adblock_allow_row);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, popups_allow_row);
 
   /* Homepage */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, homepage_box);
@@ -924,7 +928,7 @@ prefs_general_page_class_init (PrefsGeneralPageClass *klass)
 
   /* Downloads */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, download_box);
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, ask_on_download_switch);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, ask_on_download_row);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, download_folder_row);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, download_folder_label);
 
@@ -933,19 +937,18 @@ prefs_general_page_class_init (PrefsGeneralPageClass *klass)
 
   /* Session */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, session_box);
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, start_in_incognito_mode_switch);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, start_in_incognito_mode_row);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, restore_session_row);
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, restore_session_switch);
 
   /* Browsing */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, browsing_box);
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, enable_mouse_gesture_switch);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, enable_mouse_gesture_row);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, enable_switch_to_new_tab);
 
   /* Languages */
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, lang_group);
   gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, lang_listbox);
-  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, enable_spell_checking_switch);
+  gtk_widget_class_bind_template_child (widget_class, PrefsGeneralPage, enable_spell_checking_row);
 
   /* Signals */
   gtk_widget_class_bind_template_callback (widget_class, on_webapp_icon_row_activated);
@@ -991,17 +994,11 @@ setup_general_page (PrefsGeneralPage *general_page)
   /* ======================================================================== */
   /* ========================== Web Application ============================= */
   /* ======================================================================== */
-  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) == EPHY_EMBED_SHELL_MODE_APPLICATION &&
-      !ephy_is_running_inside_sandbox ()) {
-    general_page->webapp = ephy_web_application_for_profile_directory (ephy_profile_dir (),
-                                                                       EPHY_WEB_APP_NO_TMP_ICON);
-    g_assert (general_page->webapp);
-
-    if (!g_settings_get_boolean (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_SYSTEM)) {
-      prefs_general_page_update_webapp_icon (general_page, general_page->webapp->icon_path);
-      gtk_editable_set_text (GTK_EDITABLE (general_page->webapp_url_row), general_page->webapp->url);
-      gtk_editable_set_text (GTK_EDITABLE (general_page->webapp_title_row), general_page->webapp->name);
-    }
+  EphyWebApplication *webapp = ephy_shell_get_webapp (ephy_shell_get_default ());
+  if (webapp && !ephy_is_running_inside_sandbox () && !g_settings_get_boolean (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_SYSTEM)) {
+    prefs_general_page_update_webapp_icon (general_page, webapp->icon_path);
+    gtk_editable_set_text (GTK_EDITABLE (general_page->webapp_url_row), webapp->url);
+    gtk_editable_set_text (GTK_EDITABLE (general_page->webapp_title_row), webapp->name);
   }
 
   /* ======================================================================== */
@@ -1009,13 +1006,13 @@ setup_general_page (PrefsGeneralPage *general_page)
   /* ======================================================================== */
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_ADBLOCK,
-                   general_page->adblock_allow_switch,
+                   general_page->adblock_allow_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_POPUPS,
-                   general_page->popups_allow_switch,
+                   general_page->popups_allow_row,
                    "active",
                    G_SETTINGS_BIND_INVERT_BOOLEAN);
 
@@ -1086,7 +1083,7 @@ setup_general_page (PrefsGeneralPage *general_page)
 
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ASK_ON_DOWNLOAD,
-                   general_page->ask_on_download_switch,
+                   general_page->ask_on_download_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
@@ -1095,7 +1092,7 @@ setup_general_page (PrefsGeneralPage *general_page)
   /* ======================================================================== */
   g_settings_bind (settings,
                    EPHY_PREFS_START_IN_INCOGNITO_MODE,
-                   general_page->start_in_incognito_mode_switch,
+                   general_page->start_in_incognito_mode_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
@@ -1107,7 +1104,7 @@ setup_general_page (PrefsGeneralPage *general_page)
 
   g_settings_bind_with_mapping (settings,
                                 EPHY_PREFS_RESTORE_SESSION_POLICY,
-                                general_page->restore_session_switch,
+                                general_page->restore_session_row,
                                 "active",
                                 G_SETTINGS_BIND_DEFAULT,
                                 restore_session_get_mapping,
@@ -1119,7 +1116,7 @@ setup_general_page (PrefsGeneralPage *general_page)
   /* ======================================================================== */
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_MOUSE_GESTURES,
-                   general_page->enable_mouse_gesture_switch,
+                   general_page->enable_mouse_gesture_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
@@ -1134,7 +1131,7 @@ setup_general_page (PrefsGeneralPage *general_page)
   /* ======================================================================== */
   g_settings_bind (web_settings,
                    EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
-                   general_page->enable_spell_checking_switch,
+                   general_page->enable_spell_checking_row,
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
