@@ -982,7 +982,9 @@ window_cmd_show_help (GSimpleAction *action,
                       GVariant      *parameter,
                       gpointer       user_data)
 {
-  gtk_show_uri (GTK_WINDOW (user_data), "help:epiphany", GDK_CURRENT_TIME);
+  g_autoptr (GtkUriLauncher) launcher = gtk_uri_launcher_new ("help:epiphany");
+
+  gtk_uri_launcher_launch (launcher, GTK_WINDOW (user_data), NULL, NULL, NULL);
 }
 
 #define ABOUT_GROUP "About"
@@ -1347,7 +1349,6 @@ window_cmd_new_tab (GSimpleAction *action,
   ephy_link_open (EPHY_LINK (window),
                   url, NULL,
                   EPHY_LINK_NEW_TAB | EPHY_LINK_JUMP_TO);
-  ephy_window_close_pages_view (window);
   g_free (url);
 }
 
@@ -1467,7 +1468,7 @@ ephy_application_dialog_data_free (EphyApplicationDialogData *data)
     g_signal_handlers_disconnect_by_func (data->download, download_finished_cb, data);
     g_signal_handlers_disconnect_by_func (data->download, download_failed_cb, data);
 
-    g_clear_object (&data->download);
+    data->download = NULL;
   }
 
   g_cancellable_cancel (data->cancellable);
@@ -1790,7 +1791,6 @@ set_default_application_title (EphyApplicationDialogData *data,
   }
 
   if (title == NULL || title[0] == '\0') {
-    g_clear_pointer (&title, g_free);
     title = g_strdup (webkit_web_view_get_title (WEBKIT_WEB_VIEW (data->view)));
   }
 
@@ -1951,7 +1951,7 @@ static void
 create_install_dialog_when_ready (EphyApplicationDialogData *data)
 {
   XdpPortal *portal;
-  g_autoptr (XdpParent) parent = NULL;
+  XdpParent *parent;
 
   if (!data->webapp_options_set || !data->title || !data->icon_v)
     return;
@@ -2267,7 +2267,9 @@ window_cmd_cut (GSimpleAction *action,
   } else {
     EphyEmbed *embed;
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_assert (embed != NULL);
+
+    if (!embed)
+      return;
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_CUT);
   }
@@ -2287,7 +2289,9 @@ window_cmd_copy (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_assert (embed != NULL);
+
+    if (!embed)
+      return;
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_COPY);
   }
@@ -2307,7 +2311,9 @@ window_cmd_paste (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_assert (embed != NULL);
+
+    if (!embed)
+      return;
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_PASTE);
   }
@@ -2327,7 +2333,9 @@ window_cmd_paste_as_plain_text (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_assert (embed != NULL);
+
+    if (!embed)
+      return;
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), WEBKIT_EDITING_COMMAND_PASTE_AS_PLAIN_TEXT);
   }
@@ -2347,7 +2355,9 @@ window_cmd_delete (GSimpleAction *action,
     EphyEmbed *embed;
 
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-    g_assert (embed != NULL);
+
+    if (!embed)
+      return;
 
     /* FIXME: TODO */
 #if 0
@@ -2571,7 +2581,6 @@ window_cmd_select_all (GSimpleAction *action,
 
     embed = ephy_embed_container_get_active_child
               (EPHY_EMBED_CONTAINER (window));
-    g_assert (embed != NULL);
 
     webkit_web_view_execute_editing_command (EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed), "SelectAll");
   }
@@ -2584,8 +2593,11 @@ window_cmd_send_to (GSimpleAction *action,
 {
   EphyWindow *window = EPHY_WINDOW (user_data);
   EphyEmbed *embed;
-  char *command, *subject, *body;
+  g_autofree char *command = NULL;
+  g_autofree char *subject = NULL;
+  g_autofree char *body = NULL;
   const char *location, *title;
+  g_autoptr (GtkUriLauncher) launcher = NULL;
 
   embed = ephy_embed_container_get_active_child
             (EPHY_EMBED_CONTAINER (window));
@@ -2601,12 +2613,8 @@ window_cmd_send_to (GSimpleAction *action,
                          "?Subject=", subject,
                          "&Body=", body, NULL);
 
-  g_free (subject);
-  g_free (body);
-
-  gtk_show_uri (GTK_WINDOW (window), command, GDK_CURRENT_TIME);
-
-  g_free (command);
+  launcher = gtk_uri_launcher_new (command);
+  gtk_uri_launcher_launch (launcher, GTK_WINDOW (window), NULL, NULL, NULL);
 }
 
 void
@@ -2635,20 +2643,12 @@ window_cmd_go_home (GSimpleAction *action,
                   EPHY_LINK_HOME_PAGE);
 }
 
-void
-window_cmd_go_content (GSimpleAction *action,
-                       GVariant      *parameter,
-                       gpointer       user_data)
-{
-  ephy_window_close_pages_view (EPHY_WINDOW (user_data));
-}
-
 void
 window_cmd_go_tabs_view (GSimpleAction *action,
                          GVariant      *parameter,
                          gpointer       user_data)
 {
-  ephy_window_open_pages_view (EPHY_WINDOW (user_data));
+  ephy_window_toggle_tab_overview (EPHY_WINDOW (user_data));
 }
 
 static void
